/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.ant.task;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.apache.tools.ant.BuildException;
import org.ow2.petals.ant.AbstractJBIAntTask;
import org.ow2.petals.ant.JBIJMXConnectorUtil;

/**
 * Task used to list the binding components
 *
 * @author ddesjardins chamerling - eBMWebsourcing
 */
public class ListBindingComponentsTask extends AbstractJBIAntTask {

    /**
     * XML namespace
     */
    private static final String XMLNS = "http://java.sun.com/xml/ns/jbi/component-info-list";

    /**
     * If supplied, only the named binding component is reported. If the binding
     * doesn’t exist, an empty component-list report is given.
     */
    private String bindingComponentName;

    /**
     * List the Bindings that have Service Units deployed to them as part of the
     * Service Assembly deployed in the JBI Environment. Without this parameter,
     * no Service Assembly dependency is verified.
     */
    private String serviceAssemblyName;

    /**
     * List the Bindings that are dependent on the specified shared library.
     * Without this parameter, no shared library dependency will be verified.
     */
    private String sharedLibraryName;

    /**
     * Binding component state Valid state are : shutdown, stated, stopped
     */
    private String state;

    /**
     * If supplied, set the given Ant property name value to the XML version of
     * this listing task.
     */
    private String xmlOutput;

    /**
     * Creates a new instance of {@link ListBindingComponentsTask}
     *
     */
    public ListBindingComponentsTask() {
        super();
    }

    /**
     * Execute the ant task
     *
     * @throws IOException
     */
    public void doTask() throws Exception {
        if (state != null
            && !(state.trim().equalsIgnoreCase("shutdown")
                || state.trim().equalsIgnoreCase("started") || state.trim()
                .equalsIgnoreCase("stopped"))) {
            throw new BuildException("Invalid state '" + state
                + "' (Valid ones are : 'shutdown', 'started', or 'stopped')");
        }

        MBeanServerConnection connection = connector.getMBeanServerConnection();

        // Get all the binding components
        ObjectName[] bindings = (ObjectName[]) connection.getAttribute(
            JBIJMXConnectorUtil.getAdminServiceMBeanName(connection),
            "BindingComponents");

        // get service assemblies names where each assembly contains service
        // units for the given component
        // Set<String> composForSA = new TreeSet<String>();
        // if (serviceAssemblyName != null && serviceAssemblyName.length() > 0)
        // {
        // // Get getComponentsForDeployedServiceAssembly
        // Object[] objects = new Object[1];
        // objects[0] = serviceAssemblyName;
        // String[] sig = new String[1];
        // sig[0] = "java.lang.String";
        // composForSA.addAll(Arrays.asList((String[]) connection.invoke(
        // JBIJMXConnectorUtil.getDeploymentServiceMBeanName(connection),
        // "getComponentsForDeployedServiceAssembly", objects, sig)));
        // }

        // get components that depends on the given shared library
        // Set<String> compsForSL = new TreeSet<String>();
        // if (sharedLibraryName != null && sharedLibraryName.length() > 0) {
        // Object[] parameters = new Object[] {sharedLibraryName};
        // String[] signature = new String[] {"java.lang.String"};
        // compsForSL.addAll(Arrays.asList((String[]) connection
        // .invoke(JBIJMXConnectorUtil
        // .getInstallationServiceMBeanName(connection),
        // "retrieveComponentsForInstalledSharedlibrary", parameters,
        // signature)));
        // }

        List<ObjectName> components = new ArrayList<ObjectName>();
        for (ObjectName objectName : bindings) {
            String objState = (String) connection.getAttribute(objectName,
                "CurrentState");
            String name = objectName.getKeyProperty("name");

            // skip this BC if state has been provided and is not equal to the
            // current one
            if (state != null && state.length() > 0
                && !state.equalsIgnoreCase(objState)) {
                continue;
            }

            // skip this BC if name has been provided and is not equal to the
            // current one
            if (bindingComponentName != null
                && bindingComponentName.length() > 0
                && !bindingComponentName.equals(name)) {
                continue;
            }

            // skip this BC if service assembly has been provided and if the BC
            // does not use it
            if (serviceAssemblyName != null && serviceAssemblyName.length() > 0) {
                Set<String> saNames = new TreeSet<String>();
                Object[] parameters = new Object[] {name};
                String[] signature = new String[] {"java.lang.String"};
                saNames.addAll(Arrays.asList((String[]) connection.invoke(
                    JBIJMXConnectorUtil
                        .getDeploymentServiceMBeanName(connection),
                    "getDeployedServiceAssembliesForComponent", parameters,
                    signature)));

                if (!saNames.contains(serviceAssemblyName)) {
                    continue;
                }
            }

            // skip this BC if shared library has been provided and if the BC
            // does not use it
            if (sharedLibraryName != null && sharedLibraryName.length() > 0) {
                // TODO
            }

            // add BC to list
            components.add(objectName);
        }

        // display
        StringBuffer output = new StringBuffer();
        StringBuffer outputXML = new StringBuffer();

        output
            .append("#######################################################\n");
        output
            .append("#                   Binding Components                #\n");
        output
            .append("#######################################################\n");

        outputXML.append("<?xml version='1.0' encoding='utf-8'?>\n");
        outputXML.append("<component-info-list xmlns='");
        outputXML.append(ListBindingComponentsTask.XMLNS);
        outputXML.append("' version='1.0'>\n");

        for (ObjectName objectName : components) {
            String name = objectName.getKeyProperty("name");
            String description = "";
            // (String) connection.getAttribute(objectName,"Description");
            String objState = (String) connection.getAttribute(objectName,
                "CurrentState");

            // Add the component to the output
            output.append("Name        : " + name + "\n");
            output.append("State       : " + objState + "\n");
            output.append("Description : " + description + "\n\n");

            outputXML
                .append("\t<component-info type='binding-component' name='"
                    + name + "' state='" + objState + "'>\n");
            outputXML.append("\t\t<description>");
            outputXML.append(description);
            outputXML.append("</description>\n");
            outputXML.append("\t</component-info>\n");
        }
        outputXML.append("</component-info-list>");

        // display or save as property
        if (xmlOutput == null) {
            try {
                log(output.toString());
            } catch (NullPointerException e) {
            }
        } else {
            try {
                getProject().setNewProperty(xmlOutput, outputXML.toString());
            } catch (Exception e) {
            }
        }
    }

    /**
     * @param bindingComponentName
     *            the bindingComponentName to set
     */
    public void setBindingComponentName(String bindingComponentName) {
        this.bindingComponentName = bindingComponentName;
    }

    /**
     * @param serviceAssemblyName
     *            the serviceAssemblyName to set
     */
    public void setServiceAssemblyName(String serviceAssemblyName) {
        this.serviceAssemblyName = serviceAssemblyName;
    }

    /**
     * @param sharedLibraryName
     *            the sharedLibraryName to set
     */
    public void setSharedLibraryName(String sharedLibraryName) {
        this.sharedLibraryName = sharedLibraryName;
    }

    /**
     * @param state
     *            the state to set
     */
    public void setState(String state) {
        this.state = state;
    }

    /**
     * @param xmlOutput
     *            the xmlOutput to set
     */
    public void setXmlOutput(String xmlOutput) {
        this.xmlOutput = xmlOutput;
    }

}
