/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.ant.task;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.jbi.management.DeploymentServiceMBean;
import javax.jbi.management.LifeCycleMBean;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;

import org.apache.tools.ant.BuildException;
import org.ow2.petals.ant.AbstractJBIAntTask;
import org.ow2.petals.ant.JBIJMXConnectorUtil;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.descriptor.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.ServiceUnit;

/**
 * Ant task : jbi-list-service-assemblies. This task prints information about
 * service assemblies deployed in the JBI environment.
 *
 * @author ddesjardins chamerling - eBMWebsourcing
 */
public class ListServiceAssembliesTask extends AbstractJBIAntTask {

    /**
     * XML namespace
     */
    private static final String XMLNS = "http://java.sun.com/xml/ns/jbi/service-assembly-info-list";

    /**
     * List the service assemblies that has service units deployed to this
     * component. Without this parameter, all the service assemblies that are
     * deployed in the JBI Environment will be displayed.
     */
    private String componentName;

    /**
     * List the Bindings that have Service Units deployed to them as part of the
     * Service Assembly deployed in the JBI Environment. Without this parameter,
     * no Service Assembly dependency is verified.
     */
    private String serviceAssemblyName;

    /**
     * Binding component state Valid state are : shutdown, started, stopped (not
     * case sensitive).
     */
    private String state;

    /**
     * If supplied, set the given Ant property name value to the XML version of
     * this listing task.
     */
    private String xmlOutput;

    /**
     * Creates a new instance of {@link ListServiceAssembliesTask}
     *
     */
    public ListServiceAssembliesTask() {
        super();
    }

    /**
     * Execute the ant task
     *
     * @throws IOException
     */
    public void doTask() throws Exception {
        if (state != null
                && !(DeploymentServiceMBean.SHUTDOWN.equalsIgnoreCase(state.trim())
                    || DeploymentServiceMBean.STARTED.equalsIgnoreCase(state.trim())
                    || DeploymentServiceMBean.STOPPED.equalsIgnoreCase(state.trim()))) {
                throw new BuildException(
                    "Valid states are : 'Shutdown', 'Started', or 'Stopped'");
            }

        MBeanServerConnection connection = connector.getMBeanServerConnection();
        // Output
        StringBuffer output = new StringBuffer();
        StringBuffer outputXML = new StringBuffer();
        output
            .append("#######################################################\n");
        output
            .append("#                 Service Assemblies                  #\n");
        output
            .append("#######################################################\n");
        outputXML.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
        outputXML.append("<service-assembly-info-list xmlns=\"").append(
                ListServiceAssembliesTask.XMLNS)
                .append("\" version=\"1.0\">\n");
        // Get all deployed SA with state "Started", "Stopped", "Shutdown" or "Unknown"
        Object deployedSAs = connection.getAttribute(JBIJMXConnectorUtil
            .getDeploymentServiceMBeanName(connection),
            "DeployedServiceAssemblies");
        Set<String> saForCompo = new TreeSet<String>();
        if (componentName != null) {
            // Get getDeployedServiceAssembliesForComponent
            Object resultSA = connection.invoke(
                    JBIJMXConnectorUtil
                            .getDeploymentServiceMBeanName(connection),
                    "getDeployedServiceAssembliesForComponent",
                    new Object[] { componentName },
                    new String[] { "java.lang.String" });
            if (resultSA instanceof String[]) {
                saForCompo.addAll(Arrays.asList((String[]) deployedSAs));
            }
        }
        String saName = serviceAssemblyName;
        String saState = state;
        if (deployedSAs instanceof String[]) {
            String[] assemblies = (String[]) deployedSAs;
            for (String assemblyName : assemblies) {
                String assemblyState = LifeCycleMBean.UNKNOWN;
                try {
                    assemblyState = (String) connection.invoke(
                            JBIJMXConnectorUtil
                                    .getDeploymentServiceMBeanName(connection),
                            "getState",
                            new Object[] { assemblyName },
                            new String[] { "java.lang.String" });
                } catch (MBeanException e) {
                    // When an error occurs during the processing of "getState", we
                    // skip it and consider that the state is "Unkown"
                }
                // Set the variable to have a positive test
                if (state == null) {
                    saState = assemblyState;
                }
                if (serviceAssemblyName == null) {
                    saName = assemblyName;
                }
                if (componentName == null) {
                    saForCompo.add(assemblyName);
                }
                if (assemblyName.equals(saName) && assemblyState.equals(saState)
                    && saForCompo.contains(assemblyName)) {
                    
                    // Get SA Description
                    String saStringJBIDescriptor = (String) connection.invoke(
                            JBIJMXConnectorUtil
                                    .getDeploymentServiceMBeanName(connection),
                            "getServiceAssemblyDescriptor",
                            new Object[] { saName },
                            new String[] { "java.lang.String" });
                    
                    ByteArrayInputStream bais = new ByteArrayInputStream(saStringJBIDescriptor.getBytes());
                    JBIDescriptorBuilder saJbiDescriptorBuilder = JBIDescriptorBuilder.getInstance();
                    JBIDescriptor saJbiDescriptor = saJbiDescriptorBuilder.buildDescriptor(bais);
                    bais.close();
                    
                    String saDescription = saJbiDescriptor.getServiceAssembly().getIdentification().getDescription();
                    
                    // Add the component to the output
                    output.append("Name : " + assemblyName + "\n");
                    output.append("State : " + assemblyState + "\n");
                    output.append("Description : " + saDescription + "\n\n");
                    outputXML.append("\t<service-assembly-info name=\"")
                            .append(assemblyName).append("\" state=\"")
                            .append(assemblyState).append("\">\n");
                    outputXML.append("\t\t<description>").append(saDescription)
                            .append("</description>\n");

                    // get service units names for the current SA
                    Set<String> sus = new TreeSet<String>(Arrays
                        .asList((String[]) connection.invoke(
                            JBIJMXConnectorUtil
                                    .getDeploymentServiceMBeanName(connection),
                            "getServiceUnitForServiceAssembly",
                            new Object[] { assemblyName },
                            new String[] { "java.lang.String"})));

                    for (String suName : sus) {
                        // Get SU Description from the SA JBI descriptor
                        String suDescription = "";
                        String suComponentNameDeployedOn = "";
                        if ((saJbiDescriptor.getServiceAssembly() != null)
                                && (saJbiDescriptor.getServiceAssembly()
                                        .getServiceUnits() != null)) {
                            List<ServiceUnit> saJbiDescriptorSUs = saJbiDescriptor
                                    .getServiceAssembly().getServiceUnits();
                            for (ServiceUnit saJbiDescriptorSU : saJbiDescriptorSUs) {
                                if ((saJbiDescriptorSU.getIdentification() != null)
                                        && (suName.equals(saJbiDescriptorSU
                                                .getIdentification().getName()))) {
                                    suDescription = saJbiDescriptorSU
                                            .getIdentification().getDescription();
                                    suComponentNameDeployedOn = saJbiDescriptorSU
                                            .getTargetComponentName();
                                    break;
                                }
                            }
                        }
                        
                        outputXML.append("\t\t<service-unit-info name=\"").append(suName)
                                .append("\" state=\"").append(assemblyState)
                                .append("\" deployed-on=\"").append(suComponentNameDeployedOn)
                                .append("\">\n");
                        outputXML.append("\t\t\t<description>").append(
                                suDescription).append("</description>\n");
                        outputXML.append("\t\t</service-unit-info>\n");
                    }

                    outputXML.append("\t</service-assembly-info>\n");
                }
            }
            outputXML.append("</service-assembly-info-list>");
            if (xmlOutput == null) {
                try {
                    log(output.toString());
                } catch (NullPointerException e) {
                }
            } else {
                try {
                    getProject()
                        .setNewProperty(xmlOutput, outputXML.toString());
                } catch (Exception e) {
                }
            }
        }
    }

    /**
     * @param componentName
     *            the componentName to set
     */
    public void setComponentName(String componentName) {
        this.componentName = componentName;
    }

    /**
     * @param serviceAssemblyName
     *            the serviceAssemblyName to set
     */
    public void setServiceAssemblyName(String serviceAssemblyName) {
        this.serviceAssemblyName = serviceAssemblyName;
    }

    /**
     * @param state
     *            the state to set
     */
    public void setState(String state) {
        this.state = state;
    }

    /**
     * @param xmlOutput
     *            the xmlOutput to set
     */
    public void setXmlOutput(String xmlOutput) {
        this.xmlOutput = xmlOutput;
    }
}
